<?php
/**
 * @package    DOCman
 * @copyright   Copyright (C) 2011 Timble CVBA (http://www.timble.net)
 * @license     GNU GPLv3 <http://www.gnu.org/licenses/gpl.html>
 * @link        http://www.joomlatools.com
 */

class ComDocmanViewFlatHtml extends ComDocmanViewHtml
{
    protected function _initialize(KObjectConfig $config)
    {
        $config->append(array(
            'auto_fetch' => false
        ));

        parent::_initialize($config);
    }

    public function isCollection()
    {
        return true;
    }

    protected function _fetchData(KViewContext $context)
    {
        $params = $this->getParameters();

        $context->data->event_context = 'com_docman.documents';
        $context->data->documents = $this->getModel()->fetch();
        $context->data->total     = $this->getModel()->count();

        foreach ($context->data->documents as $document) {
            $this->prepareDocument($document, $params, $context->data->event_context);
        }

        parent::_fetchData($context);

        $context->parameters->total = $this->getModel()->count();
        
        $this->_setSearchFilterData($context);
        $this->_setAllDocumentsData($context);
    }

    protected function _setSearchFilterData(KViewContext $context)
    {
        $query = $this->getObject('request')->query;

        $context->data->filter = $this->getModel()->getState();

        $menu   = $this->getActiveMenu();
        $filter = $context->data->filter;

        // Unset the selected category in the search filter for default menu filter
        if ($query->has('category') && $query->category == $menu->query['category']) {
            $filter->remove('category');
        }

        $owner      = !empty($menu->query['created_by']) ? $menu->query['created_by'] : null;
        $tags       = !empty($menu->query['tag']) ? $menu->query['tag'] : array();
        $categories = !empty($menu->query['category']) ? $menu->query['category'] : array();
        $children   = isset($menu->query['category_children']) ? $menu->query['category_children'] : true;

        if (!empty($menu->query['own']) || empty($owner) || count($owner) <= 1) {
            $menu->params->set('show_owner_filter', false);
        }

        if (count($tags) === 1) {
            $menu->params->set('show_tag_filter', false);
            $menu->params->set('show_document_tags', false);
        }

        if (!$this->getObject('com://admin/docman.model.entity.config')->connectAvailable()) {
            $menu->params->set('show_content_filter', false);
        }
        elseif ($filter->search_contents === null) {
            $filter->search_contents = true;
        }

        $category_filter = array(
            'page'         => $this->getModel()->getState()->page,
            'access'       => $this->getObject('user')->getRoles(),
            'current_user' => $this->getObject('user')->getId(),
            'enabled'      => true
        );

        if ($categories)
        {
            if ($children) {
                $category_filter['parent_id'] = $categories;
                $category_filter['include_self'] = true;
            } else {
                $category_filter['id'] = $categories;
            }
        }

        $context->data->filter_toggled = ($context->parameters->total > $this->getModel()->getState()->limit)
        || (!empty($filter->search)
            || (!empty($filter->category) && $filter->category != $categories)
            || (!empty($filter->tag) && $filter->tag != $tags)
            || (!empty($filter->created_by) && $filter->created_by != $owner));

        $context->data->category_filter = $category_filter;

        $context->data->tag_model  = $this->getObject('com://admin/docman.model.pagetags');
        $context->data->tag_filter = [
            'page'         => $this->getModel()->getState()->page,
            'access'       => $this->getObject('user')->getRoles(),
            'current_user' => $this->getObject('user')->getId(),
            'enabled'      => $this->getModel()->getState()->enabled,
            'status'       => $this->getModel()->getState()->status
        ];

        if (!$context->data->tag_model->setState($context->data->tag_filter->toArray())->count()) {
            $menu->params->set('show_tag_filter', false);
        }

        // Hide owner filter to non-managers
        $menu->params->set('show_owner_filter', $this->getObject('user')->authorise('core.manage', 'com_docman'));
    }

    /**
     * Fetch all documents and prepare it for use with slideshow (Photoswipe)
     *
     * @param KViewContext $context
     * @return void
     */
    protected function _setAllDocumentsData(KViewContext $context)
    {
        $params = $this->getParameters();
        $states = $this->getModel()->getState()->getValues();

        // Fetch all documents, discarding pagination
        if (isset($states['limit'])) {
            unset($states['limit']);
        }

        $model = $this->getObject('com:docman.model.documents');
        $model->getState()->setValues($states);
        $documents = $model->fetch();

        // Append slideshow-specific details
        foreach ($documents as $document)
        {
            $this->prepareDocument($document, $params, $context->data->event_context);
            $document->w = $document->storage->width;
            $document->h = $document->storage->height;
            $document->src = (string) $document->title_link;
        }

        $documents = $documents->toArray();
        $new_documents = [];

        // Workaround to force array keys to be string to prevent browser's automatic reordering of js array items
        array_walk($documents, function ($v, $k) use (&$new_documents) {
            $v = array(
                'id'    => $v['id'],
                'title' => $v['title'],
                'src'   => $v['src'],
                'w'     => $v['w'],
                'h'     => $v['h']
            );
            $new_documents[" {$k}"] = $v;
        }, ARRAY_FILTER_USE_BOTH);

        $context->data->all_documents = $new_documents;
    }
}
